
from . import tokenize
import pandas as pd
import re
from nltk import pos_tag
from konlpy.tag import Hannanum


class PosTag():

    def __init__(self, text, lang):
        self.text = text
        self.lang = lang
        self.postags = []

    def convert_to_postagdf(self):
        df = pd.DataFrame({'postags':self.postags})
        df['word'] = df.postags.apply(lambda x: x[0])
        df['pos'] = df.postags.apply(lambda x: x[1])
        del(df['postags'])
        self.postagdf = df

    def deduplicate(self, df):
        df = df.sort_values('word')
        df['lower'] = df.word.apply(lambda x: x.lower())
        return df.drop_duplicates(keep='first', subset=['lower'])

    def get_nouns(self):
        if len(self.postagdf) is 0:
            return []
        else:
            df = self.postagdf
            df1 = df[ df.pos.str.contains(pat=r'^N') ]
            # 중복제거.
            df1 = self.deduplicate(df1)
            return list(df1.word)

    # 명사에서 일반화하자.
    def get_words(self, pos_pat=r'^N'):
        if len(self.postagdf) is 0:
            self.words = []
        else:
            df = self.postagdf
            df1 = df[ df.pos.str.contains(pat=pos_pat) ]
            # 중복제거.
            words = list(df1.word)
            if pos_pat is r'^N':
                words = [w.capitalize() for w in words]
            self.words = list(set(words))

    def remove_useless_words_and_get_nouns(self):
        self.convert_to_postagdf()
        t = tokenize.TokenRemover(self.lang)
        self.postagdf = t.remove_in_df(self.postagdf)
        return self.get_nouns()

class NLTK(PosTag):

    def __init__(self, text, lang='latin'):
        super().__init__(text, lang)

    def postag(self):
        t = tokenize.Tokenizer(self.text)
        t.word_tokenize()
        self.postags = pos_tag(t.tokens)
        return self

class Konlpy(PosTag):

    def __init__(self, text, lang='korean'):
        super().__init__(text, lang)

    def postag(self):
        # 토큰화 + 품사태깅.
        hannanum = Hannanum()
        self.postags = hannanum.pos(self.text)
        return self




from . import pos
import pandas as pd
import re
from konlpy.tag import Okt




class LatinHangulSpliter:

    def __init__(self, text):
        self.text = text
        self.latin_pat = '[a-zA-Z]+'
        self.parse_latin()
        self.parse_hangul()

    def parse_latin(self):
        strings = re.findall(pattern=self.latin_pat, string=self.text)
        self.latin = " ".join(strings)

    def parse_hangul(self):
        new_string, number_of_subs_made = re.subn(pattern=self.latin_pat, repl='', string=self.text)
        self.number_of_subs_made = number_of_subs_made
        new_string = new_string.strip().lstrip()
        self.hangul = self.remove_trace(new_string)

    def remove_trace(self, text):
        new_string, number_of_subs_made = re.subn(pattern='\(\s+\)', repl='', string=text)
        return new_string

class LatinHangulSpliter_v1(pos.PosTag):

    def __init__(self, text):
        super().__init__()
        self.text = text
        okt = Okt()
        self.postags = okt.pos(self.text)
        self.split_postags()
        df = self.postagdf
        self.latin = df.query('pos == "Alpha"')
        self.hangul = df.query('pos == "Foreign"')




import unittest
from inlp.pos import *
from inlp import tokenize
import copy






@unittest.skip("showing class skipping")
class KonlpyTestCase(unittest.TestCase):

    #@unittest.skip("demonstrating skipping")
    def test_postag(self):
        print(f"\n{'='*60}\n {__class__.__name__} : {inspect.stack()[0][3]}\n")
        text = "[나꼼수 레전드편] 나꼼수 나와서 털리는 홍준표 (김어준,주진우,정봉주,김용민)__[이명박]"
        k = Konlpy(text, lang='korean')
        k.postag()
        print(f"\n\n k.postags :\n\n{k.postags}")

    #@unittest.skip("demonstrating skipping")
    def test_remove_useless_words_and_get_nouns(self):
        print(f"\n{'='*60}\n {__class__.__name__} : {inspect.stack()[0][3]}\n")
        text = "[나꼼수 레전드편] 나꼼수 나와서 털리는 홍준표 (김어준,주진우,정봉주,김용민)__[이명박]"
        k = Konlpy(text, lang='korean')
        nouns = k.postag().remove_useless_words_and_get_nouns()
        print(f"\n\n k.postagdf :\n\n{k.postagdf}")
        print(f"\n\n nouns :\n\n{nouns}")

@unittest.skip("showing class skipping")
class NLTKTestCase(unittest.TestCase):

    text = "An A-Z Index of the Apple macOS (OSX) command line - SS64 Command line reference"

    @unittest.skip("demonstrating skipping")
    def test_remove_useless_words_and_get_nouns(self):
        print(f"\n{'='*60}\n {__class__.__name__} : {inspect.stack()[0][3]}\n")
        n = NLTK(self.text, lang='latin')
        nouns = n.postag().remove_useless_words_and_get_nouns()
        print(f"\n\n n.postagdf :\n\n{n.postagdf}")
        print(f"\n\n nouns :\n\n{nouns}")

    #@unittest.skip("demonstrating skipping")
    def test_deduplicate(self):
        print(f"\n{'='*60}\n {__class__.__name__} : {inspect.stack()[0][3]}\n")
        n = NLTK(self.text, lang='latin')
        nouns = n.postag()
        n.convert_to_postagdf()
        print(f"\n\n n.postagdf :\n\n{n.postagdf}")
        t = tokenize.TokenRemover(n.lang)
        n.postagdf = t.process_in_df(n.postagdf)
        print(f"\n\n n.postagdf :\n\n{n.postagdf}")
        df = n.deduplicate(n.postagdf)
        print(f"\n df :\n\n{df}\n")







def main():
    unittest.main()




from nltk.tokenize import TweetTokenizer
from nltk.corpus import stopwords
from nltk import word_tokenize, pos_tag, DefaultTagger
# 강제 태거.
default_tagger = DefaultTagger('NN')
import string
import re
from collections import Counter
import pandas as pd
import inspect


PYTHON_REGEX_METACHARS = ['.', '^', '$', '*', '+', '?', '{', '}', '[', ']', '\\', '|', '(', ')']
SPECIAL_CHARS = [
    '.', '…', '¿', '？', '¡', '！', '–', '·',#기본
    '（', '）', '《', '》', '】', '【',#괄호
    '′', '“', '”', "‘", "’",#따옴표
    '»','—', '―', '•',
    '｜', '|', ':', ':]',#구분자
]
SHAPE_CHARS = ['▶', '❤', '🏾',]
ADDI_STOPWORDS = ['vs','rt','via']

class Tokenizer:
    """토큰화 함수 자체는 영향을 미치지 못한다."""
    def __init__(self, text):
        self.text = text.replace('__', ', ')

    def word_tokenize(self):
        self.tokens = word_tokenize(self.text)
        return self.tokens

    def tweet_tokenize(self):
        self.tokens = TweetTokenizer().tokenize(self.text)
        return self.tokens

    # 최악의 결과
    def split_tokenize(self):
        self.tokens = self.text.split()
        return self.tokens

class PatternRemover:

    def __init__(self):
        text_removal_regexs = [
            '\.\.+|\.\s(\.\s)+',# 목차에서 페이지 번호 표시를 위한 연결된 점들.
            '\d+\.\d+|\d+\.\d+(\.\d+)+',# 목차에서 문서구조를 나타내는 번호트리.
        ]
        self.text_removal_regex = "|".join(text_removal_regexs)
        #self.ToC_numbering_regex = '\d+\.\d+'

        self.tokens = None
        self.token_removal_regexs = []
        self.specialchars = SPECIAL_CHARS.copy()
        self.metachars = PYTHON_REGEX_METACHARS.copy()
        self.produce_specialchars_regexs()

    def produce_specialchars_regexs(self):
        for sc in self.specialchars:
            if sc in self.metachars: sc = "\\" + sc
            else: sc += '+'
            self.token_removal_regexs.append(sc)

    def add_text_removal_regex(self, regex):
        self.text_removal_regex += ('|' + regex)

    def add_token_removal_regex(self, regex):
        self.token_removal_regexs.append(regex)

    def remove_matched_text(self):
        new_string, number_of_subs_made = re.subn(pattern=self.text_removal_regex, repl='', string=text)
        #print(f"\n\n new_string : {new_string}")
        return new_string

    def remove_matched_tokens(self):
        if self.tokens is not None:
            refined_tokens = []
            for regex in self.token_removal_regexs:
                p = re.compile(pattern=regex)
                for tok in self.tokens:
                    if p.match(string=tok) is None:
                        refined_tokens.append(tok)
            self.tokens = refined_tokens

class TokenRemover:
    """정확한 값에 의해서 제거"""
    def __init__(self, lang):
        self.lang = lang
        self.tokens = []
        self.stopwords = get_stopwords(self.lang)
        self.addi_stopwords = ['vs','rt','via']
        self.metachars = PYTHON_REGEX_METACHARS
        self.specialchars = SPECIAL_CHARS
        self.allwords = self.stopwords + self.addi_stopwords + self.metachars + self.specialchars
        self.TF = True
        self.leaving_single_chars = ['R']

    def remove_in_li(self, tokens):
        tokens = self.remove_digits(tokens)
        tokens = self.remove_allwords(tokens)
        tokens = self.remove_non_alphabets(tokens)
        tokens = self.remove_single_chars(tokens)
        return tokens

    def remove_in_df(self, df):
        df = self.remove_digits_in_df(df)
        df = self.remove_allwords_in_df(df)
        df = self.remove_non_alphabets_in_df(df)
        df = self.remove_single_chars_in_df(df)
        return df

    def remove_stopwords(self, tokens):
        return [tok for tok in tokens if tok not in self.stopwords]

    def remove_digits(self, tokens):
        return [tok for tok in tokens if tok.isdigit() is False]

    def remove_digits_in_df(self, df):
        if len(df) is 0: return df
        else: return df[ ~df.word.str.isdigit() ]

    def remove_non_alphabets(self, tokens):
        return [tok for tok in tokens if tok.isalpha() is True]

    def remove_non_alphabets_in_df(self, df):
        if len(df) is 0: return df
        else: return df[ df.word.str.isalpha() ]

    def remove_addi_stopwords(self, tokens):
        return [tok for tok in tokens if tok not in self.addi_stopwords]

    def remove_metachars(self, tokens):
        return [tok for tok in tokens if tokc not in self.metachars]

    def remove_specialchars(self, tokens):
        return [tok for tok in tokens if tok not in self.specialchars]

    def remove_allwords(self, tokens):
        return [tok for tok in tokens if tok not in self.allwords]

    def remove_allwords_in_df(self, df):
        if len(df) is 0: return df
        else: return df[ ~df.word.isin(self.allwords) ]

    def remove_single_chars(self, tokens):
        new_tokens = []
        for tok in tokens:
            if (len(tok) is 1) and (tok not in self.leaving_single_chars):
                pass
            else:
                new_tokens.append(tok)
        return new_tokens

    def remove_single_chars_in_df(self, df):
        if len(df) is 0: return df
        else:
            df['TF'] = df.word.apply(lambda x: True if len(x) is 1 else False)
            return df.query('TF == False')

def get_stopwords(lang='english'):
    if lang is 'korean':
        words = [
            '은','는','이','가',
            '을','를',
            '이','그','저',
            '에','에서','으로','으로부터','로부터',
        ]
    elif lang is 'latin':
        words = stopwords.words('english') + stopwords.words('spanish')
    else:
        words = stopwords.words(lang)
    return list(string.punctuation) + words

class TermFrequency:

    def report(text, lang='english'):
        t = Tokenizer(text)
        tokens = t.tweet_tokenize()
        tr = TokenRemover(lang)
        tokens = tr.process(tokens)
        c = Counter()
        c.update(tokens)
        # term-frequencies
        tfs = []
        for tag, count in c.most_common(len(c)):
            tfs.append({'word':tag, 'freq':count})

        df = pd.DataFrame(tfs).sort_values(by='freq', ascending=False)
        print(f"\n\n{'*'*60}\n\n {__class__.__name__} : {inspect.stack()[0][3]} \n\n{'*'*60}\n\n")
        print(f"\n df :\n\n{df}")
        return df




import unittest
from inlp.tokenize import *




#@unittest.skip("showing class skipping")
class TokenRemoverTestCase(unittest.TestCase):


    #@unittest.skip("demonstrating skipping")
    def test_remove_single_chars(self):
        print(f"\n{'='*60}\n {__class__.__name__} : {inspect.stack()[0][3]}\n")
        tokens = ['Ñ','®','Ó',"À","Á","Ä","É","Ë","Í","Ò","Ó","Ú","Ü","前","개"]
        t = TokenRemover(lang='english')
        tokens = t.remove_single_chars(tokens)
        print(f"\n\n tokens : {tokens}")
        self.assertEqual(len(tokens), 0)

    #@unittest.skip("demonstrating skipping")
    def test_remove_single_chars__left_exceptions(self):
        print(f"\n{'='*60}\n {__class__.__name__} : {inspect.stack()[0][3]}\n")
        tokens = ['Ñ','®','Ó','R']
        t = TokenRemover(lang='english')
        tokens = t.remove_single_chars(tokens)
        print(f"\n\n tokens : {tokens}")
        self.assertEqual(len(tokens), 1)

    #@unittest.skip("demonstrating skipping")
    def test_process(self):
        print(f"\n{'='*60}\n {__class__.__name__} : {inspect.stack()[0][3]}\n")
        tokens = ['R','__','--_#_---2.','Ñ','®','Ó',"À","Á","Ä","É","Ë","Í","Ò","Ó","Ú","Ü","前","개"]
        t = TokenRemover(lang='english')
        tokens = t.process(tokens)
        print(f"\n\n tokens : {tokens}")
        self.assertEqual(len(tokens), 1)

    #@unittest.skip("demonstrating skipping")
    def test_remove_single_chars_in_df(self):
        print(f"\n{'='*60}\n {__class__.__name__} : {inspect.stack()[0][3]}\n")
        df = pd.DataFrame({'word':['이런','젠장','쉣']})
        t = TokenRemover(lang='english')
        df = t.remove_single_chars_in_df(df)
        print(f"\n\n df :\n\n{df}")
        self.assertEqual(len(df), 2)






def main():
    unittest.main()
